@@ -1,6 +1,6 @@
 /* bsd_mutex.cc
 
-   Copyright 2003 Red Hat Inc.
+   Copyright 2003, 2004 Red Hat Inc.
 
 This file is part of Cygwin.
 
@@ -9,7 +9,7 @@ Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 #ifdef __OUTSIDE_CYGWIN__
 #include "woutsup.h"
-#include "cygerrno.h"
+#include <errno.h>
 #define _KERNEL 1
 #define __BSD_VISIBLE 1
 #include <sys/smallprint.h>
@@ -180,9 +180,7 @@ _msleep (void *ident, struct mtx *mtx, int priority,
   int ret = -1;
   char name[64];
   msleep_event_name (ident, name);
-  HANDLE evt = OpenEvent (EVENT_ALL_ACCESS, FALSE, name);
-  if (!evt)
-    evt = CreateEvent (NULL, TRUE, FALSE, name);
+  HANDLE evt = CreateEvent (NULL, TRUE, FALSE, name);
   if (!evt)
     panic ("CreateEvent in msleep (%s) failed: %E", wmesg);
   if (mtx)
@@ -201,7 +199,6 @@ _msleep (void *ident, struct mtx *mtx, int priority,
   if ((priority & PCATCH)
       && td->client->signal_arrived () != INVALID_HANDLE_VALUE)
     obj_cnt = 4;
-  td->client->release ();
   switch (WaitForMultipleObjects (obj_cnt, obj, FALSE, timo ?: INFINITE))
     {
       case WAIT_OBJECT_0:	/* wakeup() has been called. */
@@ -223,10 +220,16 @@ _msleep (void *ident, struct mtx *mtx, int priority,
 	panic ("wait in msleep (%s) failed, %E", wmesg);
 	break;
     }
+#if 0
   /* Dismiss event before entering mutex. */
+  /* CV 2004-09-06, Don't dismiss for now. 
+     TODO: Dismissing was meant to solve a problem with heavy load but
+     there's no proof that it helps.  On the contrary, it breaks msgtest
+     in the testsuite.  As long as I don't get a testcase to track that
+     down, I'll keep it that way. */
   ResetEvent (evt);
+#endif
   CloseHandle (evt);
-  td->client->hold ();
   set_priority (old_priority);
   if (mtx && !(priority & PDROP))
     mtx_lock (mtx);
