@@ -1,6 +1,6 @@
 /* process.cc
 
-   Copyright 2001, 2002 Red Hat Inc.
+   Copyright 2001, 2002, 2003, 2004 Red Hat Inc.
 
    Written by Robert Collins <rbtcollins@hotmail.com>
 
@@ -10,6 +10,7 @@ This software is a copyrighted work licensed under the terms of the
 Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
+#ifdef __OUTSIDE_CYGWIN__
 #include "woutsup.h"
 
 #include <sys/types.h>
@@ -17,8 +18,6 @@ details. */
 #include <assert.h>
 #include <stdlib.h>
 
-#include "cygerrno.h"
-
 #include "process.h"
 
 /*****************************************************************************/
@@ -70,10 +69,12 @@ process::process (const pid_t cygpid, const DWORD winpid, HANDLE signal_arrived)
 		       GetLastError ());
     }
   InitializeCriticalSection (&_access);
+  debug ("initialized (%lu)", _cygpid);
 }
 
 process::~process ()
 {
+  debug ("deleting (%lu)", _cygpid);
   DeleteCriticalSection (&_access);
   CloseHandle (_signal_arrived);
   CloseHandle (_hProcess);
@@ -106,7 +107,7 @@ process::add (cleanup_routine *const entry)
   assert (entry);
 
   bool res = false;
-  EnterCriticalSection (&_access);
+  hold ();
 
   if (!_cleaning_up)
     {
@@ -115,7 +116,7 @@ process::add (cleanup_routine *const entry)
       res = true;
     }
 
-  LeaveCriticalSection (&_access);
+  release ();
   return res;
 }
 
@@ -125,7 +126,7 @@ process::remove (const cleanup_routine *const entry)
   assert (entry);
 
   bool res = false;
-  EnterCriticalSection (&_access);
+  hold ();
 
   if (!_cleaning_up)
     {
@@ -149,7 +150,7 @@ process::remove (const cleanup_routine *const entry)
 	}
     }
 
-  LeaveCriticalSection (&_access);
+  release ();
   return res;
 }
 
@@ -160,13 +161,13 @@ process::remove (const cleanup_routine *const entry)
 void
 process::cleanup ()
 {
-  EnterCriticalSection (&_access);
+  hold ();
   assert (!is_active ());
   assert (!_cleaning_up);
   InterlockedExchange (&_cleaning_up, true);
   cleanup_routine *entry = _routines_head;
   _routines_head = NULL;
-  LeaveCriticalSection (&_access);
+  release ();
 
   while (entry)
     {
@@ -250,7 +251,6 @@ process_cache::process (const pid_t cygpid, const DWORD winpid,
 			  "new connection refused for %d(%lu)"),
 			 MAXIMUM_WAIT_OBJECTS - SPECIALS_COUNT,
 			 cygpid, winpid);
-	  set_errno (EAGAIN);
 	  return NULL;
 	}
 
@@ -259,7 +259,6 @@ process_cache::process (const pid_t cygpid, const DWORD winpid,
 	{
 	  LeaveCriticalSection (&_cache_write_access);
 	  delete entry;
-	  set_errno (ESRCH);
 	  return NULL;
 	}
 
@@ -278,7 +277,7 @@ process_cache::process (const pid_t cygpid, const DWORD winpid,
       SetEvent (_cache_add_trigger);
     }
 
-  EnterCriticalSection (&entry->_access); // To be released by the caller.
+  entry->hold (); // To be released by the caller.
   LeaveCriticalSection (&_cache_write_access);
   assert (entry);
   assert (entry->_winpid == winpid);
@@ -433,3 +432,4 @@ process_cache::find (const DWORD winpid, class process **previous)
 }
 
 /*****************************************************************************/
+#endif /* __OUTSIDE_CYGWIN__ */
