@@ -98,7 +98,6 @@ extern "C"
   char ***main_environ;
   /* __progname used in getopt error message */
   char *__progname;
-  struct _reent reent_data;
   struct per_process __cygwin_user_data =
   {/* initial_sp */ 0, /* magic_biscuit */ 0,
    /* dll_major */ CYGWIN_VERSION_DLL_MAJOR,
@@ -119,7 +118,7 @@ extern "C"
    /* api_minor */ CYGWIN_VERSION_API_MINOR,
    /* unused2 */ {0, 0, 0, 0, 0},
    /* resourcelocks */ &_reslock, /* threadinterface */ &_mtinterf,
-   /* impure_ptr */ &reent_data,
+   /* impure_ptr */ _GLOBAL_REENT,
   };
   bool ignore_case_with_glob;
   int __declspec (dllexport) _check_for_executable = true;
@@ -605,7 +604,6 @@ dll_crt0_0 ()
   GetStartupInfo (&si);
   child_proc_info = (child_info *) si.lpReserved2;
 
-  int mypid = 0;
   if (si.cbReserved2 < EXEC_MAGIC_SIZE || !child_proc_info
       || memcmp (child_proc_info->zero, zeros,
 		 sizeof (child_proc_info->zero)) != 0)
@@ -625,7 +623,7 @@ dll_crt0_0 ()
       switch (child_proc_info->type)
 	{
 	  case _PROC_FORK:
-	    user_data->forkee = child_proc_info->cygpid;
+	    user_data->forkee = true;
 	    should_be_cb = sizeof (child_info_fork);
 	    /* fall through */;
 	  case _PROC_SPAWN:
@@ -639,7 +637,6 @@ dll_crt0_0 ()
 	    else
 	      {
 		cygwin_user_h = child_proc_info->user_h;
-		mypid = child_proc_info->cygpid;
 		break;
 	      }
 	  default:
@@ -667,7 +664,7 @@ dll_crt0_0 ()
 	    alloc_stack (fork_info);
 	    cygheap_fixup_in_child (false);
 	    memory_init ();
-	    set_myself (mypid);
+	    set_myself (NULL);
 	    close_ppid_handle = !!child_proc_info->pppid_handle;
 	    break;
 	  case _PROC_SPAWN:
@@ -686,12 +683,11 @@ dll_crt0_0 ()
 				  hMainProc, &h, 0, FALSE,
 				  DUPLICATE_SAME_ACCESS | DUPLICATE_CLOSE_SOURCE))
 	      h = NULL;
-	    set_myself (mypid, h);
+	    set_myself (h);
 	    __argc = spawn_info->moreinfo->argc;
 	    __argv = spawn_info->moreinfo->argv;
 	    envp = spawn_info->moreinfo->envp;
 	    envc = spawn_info->moreinfo->envc;
-	    envp = spawn_info->moreinfo->envp;
 	    cygheap->fdtab.fixup_after_exec ();
 	    signal_fixup_after_exec ();
 	    if (spawn_info->moreinfo->old_title)
@@ -771,7 +767,7 @@ dll_crt0_1 (char *)
 	  _tlsbase = (char *) fork_info->stackbottom;
 	  _tlstop = (char *) fork_info->stacktop;
 	}
-      longjmp (fork_info->jmp, fork_info->cygpid);
+      longjmp (fork_info->jmp, true);
     }
 
 #ifdef DEBUGGING
@@ -927,14 +923,14 @@ _dll_crt0 ()
   *main_environ = NULL;
 
   char padding[CYGTLS_PADSIZE];
-  _impure_ptr = &reent_data;
+  _impure_ptr = _GLOBAL_REENT;
   _impure_ptr->_stdin = &_impure_ptr->__sf[0];
   _impure_ptr->_stdout = &_impure_ptr->__sf[1];
   _impure_ptr->_stderr = &_impure_ptr->__sf[2];
   _impure_ptr->_current_locale = "C";
 
   if (child_proc_info && child_proc_info->type == _PROC_FORK)
-    user_data->forkee = child_proc_info->cygpid;
+    user_data->forkee = true;
   else
     __sinit (_impure_ptr);
 
@@ -949,7 +945,7 @@ dll_crt0 (per_process *uptr)
   if (uptr && uptr != user_data)
     {
       memcpy (user_data, uptr, per_process_overwrite);
-      *(user_data->impure_ptr_ptr) = &reent_data;
+      *(user_data->impure_ptr_ptr) = _GLOBAL_REENT;
     }
   _dll_crt0 ();
 }
