@@ -593,7 +593,8 @@ extern "C" {
 static void
 sig_handle_tty_stop (int sig)
 {
-  /* Silently ignore attempts to suspend if there is no accomodating
+  _my_tls.incyg = 1;
+  /* Silently ignore attempts to suspend if there is no accommodating
      cygwin parent to deal with this behavior. */
   if (!myself->ppid_handle)
     {
@@ -633,6 +634,7 @@ sig_handle_tty_stop (int sig)
       api_fatal ("WaitSingleObject failed, %E");
       break;
     }
+  _my_tls.incyg = 0;
   return;
 }
 }
@@ -805,6 +807,8 @@ ctrl_c_handler (DWORD type)
 
   if (!cygwin_finished_initializing)
     {
+      if (myself->ppid_handle)	/* Was this process created by a cygwin process? */
+	return TRUE;		/* Yes.  Let the parent eventually handle CTRL-C issues. */
       debug_printf ("exiting with status %p", STATUS_CONTROL_C_EXIT);
       ExitProcess (STATUS_CONTROL_C_EXIT);
     }
