@@ -324,6 +324,7 @@ class fhandler_base
   bool is_fs_special () {return pc.is_fs_special ();}
   bool device_access_denied (int) __attribute__ ((regparm (2)));
   int fhaccess (int flags) __attribute__ ((regparm (2)));
+  friend class fhandler_fifo;
 };
 
 class fhandler_socket: public fhandler_base
@@ -454,6 +455,7 @@ class fhandler_fifo: public fhandler_pipe
   HANDLE owner;		// You can't have too many mutexes, now, can you?
   long read_use;
   long write_use;
+  virtual HANDLE& get_io_handle () { return io_handle ?: output_handle; }
 public:
   fhandler_fifo ();
   int open (int flags, mode_t mode = 0);
@@ -480,22 +482,19 @@ class fhandler_dev_raw: public fhandler_base
     unsigned eom_detected    : 1;
     unsigned eof_detected    : 1;
     unsigned lastblk_to_read : 1;
-    unsigned is_writing      : 1;
    public:
     status_flags () :
-      eom_detected (0), eof_detected (0), lastblk_to_read (0), is_writing (0)
+      eom_detected (0), eof_detected (0), lastblk_to_read (0)
       {}
   } status;
  
   IMPLEMENT_STATUS_FLAG (bool, eom_detected)
   IMPLEMENT_STATUS_FLAG (bool, eof_detected)
   IMPLEMENT_STATUS_FLAG (bool, lastblk_to_read)
-  IMPLEMENT_STATUS_FLAG (bool, is_writing)
 
   virtual BOOL write_file (const void *buf, DWORD to_write,
 			   DWORD *written, int *err);
   virtual BOOL read_file (void *buf, DWORD to_read, DWORD *read, int *err);
-  virtual int writebuf (void);
 
   /* returns not null, if `win_error' determines an end of media condition */
   virtual int is_eom(int win_error);
@@ -533,7 +532,6 @@ class fhandler_dev_floppy: public fhandler_dev_raw
   fhandler_dev_floppy ();
 
   virtual int open (int flags, mode_t mode = 0);
-  virtual int close (void);
 
   virtual _off64_t lseek (_off64_t offset, int whence);
 
@@ -1071,12 +1069,10 @@ class fhandler_dev_dsp: public fhandler_base
   int audiofreq_;
   int audiobits_;
   int audiochannels_;
-  static int open_count; // per process
   Audio_out *audio_out_;
   Audio_in  *audio_in_;
  public:
   fhandler_dev_dsp ();
-  ~fhandler_dev_dsp();
 
   int open (int flags, mode_t mode = 0);
   int write (const void *ptr, size_t len);
@@ -1088,6 +1084,9 @@ class fhandler_dev_dsp: public fhandler_base
   void dump (void);
   void fixup_after_fork (HANDLE parent);
   void fixup_after_exec ();
+ private:
+  void close_audio_in ();
+  void close_audio_out (bool immediately = false);
 };
 
 class fhandler_virtual : public fhandler_base
