@@ -392,14 +392,14 @@ format_proc_meminfo (char *destbuf, size_t maxsize)
   mem_free = memory_status.dwAvailPhys;
   PSYSTEM_PAGEFILE_INFORMATION spi = NULL;
   ULONG size = 512;
-  NTSTATUS ret;
+  NTSTATUS ret = STATUS_SUCCESS;
   spi = (PSYSTEM_PAGEFILE_INFORMATION) malloc (size);
   if (spi)
     {
       ret = NtQuerySystemInformation (SystemPagefileInformation, (PVOID) spi,
 				      size, &size);
       if (ret == STATUS_INFO_LENGTH_MISMATCH)
-        {
+	{
 	  free (spi);
 	  spi = (PSYSTEM_PAGEFILE_INFORMATION) malloc (size);
 	  if (spi)
@@ -416,7 +416,7 @@ format_proc_meminfo (char *destbuf, size_t maxsize)
     {
       PSYSTEM_PAGEFILE_INFORMATION spp = spi;
       do
-        {
+	{
 	  swap_total += spp->CurrentSize * getpagesize ();
 	  swap_free += (spp->CurrentSize - spp->TotalUsed) * getpagesize ();
 	}
@@ -465,8 +465,7 @@ format_proc_uptime (char *destbuf, size_t maxsize)
       else if (ret != STATUS_SUCCESS)
 	{
 	  __seterrno_from_win_error (RtlNtStatusToDosError (ret));
-	  debug_printf("NtQuerySystemInformation: ret = %d, "
-		       "Dos(ret) = %d",
+	  debug_printf("NtQuerySystemInformation: ret %d, Dos(ret) %d",
 		       ret, RtlNtStatusToDosError (ret));
 	  return 0;
 	}
@@ -503,8 +502,7 @@ format_proc_stat (char *destbuf, size_t maxsize)
 	  != STATUS_SUCCESS)
 	{
 	  __seterrno_from_win_error (RtlNtStatusToDosError (ret));
-	  debug_printf ("NtQuerySystemInformation: ret = %d, "
-			"Dos(ret) = %d",
+	  debug_printf ("NtQuerySystemInformation: ret %d, Dos(ret) %d",
 			ret, RtlNtStatusToDosError (ret));
 	  sbi.NumberProcessors = 1;
 	}
@@ -546,8 +544,7 @@ format_proc_stat (char *destbuf, size_t maxsize)
       if (ret != STATUS_SUCCESS)
 	{
 	  __seterrno_from_win_error (RtlNtStatusToDosError (ret));
-	  debug_printf("NtQuerySystemInformation: ret = %d, "
-		       "Dos(ret) = %d",
+	  debug_printf("NtQuerySystemInformation: ret %d, Dos(ret) %d",
 		       ret, RtlNtStatusToDosError (ret));
 	  return 0;
 	}
@@ -698,6 +695,14 @@ format_proc_cpuinfo (char *destbuf, size_t maxsize)
 	  cpuid (&maxf, &vendor_id[0], &vendor_id[2], &vendor_id[1], 0);
 	  maxf &= 0xffff;
 	  vendor_id[3] = 0;
+
+	  // vendor identification
+	  bool is_amd = false, is_intel = false;
+	  if (!strcmp ((char*)vendor_id, "AuthenticAMD"))
+	    is_amd = true;
+	  else if (!strcmp ((char*)vendor_id, "GenuineIntel"))
+	    is_intel = true;
+
 	  bufptr += __small_sprintf (bufptr, "vendor_id       : %s\n", (char *)vendor_id);
 	  unsigned cpu_mhz  = 0;
 	  if (wincap.is_winnt ())
@@ -711,13 +716,13 @@ format_proc_cpuinfo (char *destbuf, size_t maxsize)
 	      cpuid (&cpuid_sig, &extra_info, &features2, &features1, 1);
 	      /* unsigned extended_family = (cpuid_sig & 0x0ff00000) >> 20,
 			  extended_model  = (cpuid_sig & 0x000f0000) >> 16; */
-	      unsigned type            = (cpuid_sig & 0x00003000) >> 12,
-		       family          = (cpuid_sig & 0x00000f00) >> 8,
-		       model           = (cpuid_sig & 0x000000f0) >> 4,
-		       stepping        = cpuid_sig & 0x0000000f;
-	      unsigned brand_id        = extra_info & 0x0000000f,
-		       cpu_count       = (extra_info & 0x00ff0000) >> 16,
-		       apic_id         = (extra_info & 0xff000000) >> 24;
+	      unsigned type		= (cpuid_sig & 0x00003000) >> 12,
+		       family		= (cpuid_sig & 0x00000f00) >> 8,
+		       model		= (cpuid_sig & 0x000000f0) >> 4,
+		       stepping		= cpuid_sig & 0x0000000f;
+	      unsigned brand_id		= extra_info & 0x0000000f,
+		       cpu_count	= (extra_info & 0x00ff0000) >> 16,
+		       apic_id		= (extra_info & 0xff000000) >> 24;
 	      const char *type_str;
 	      switch (type)
 		{
@@ -833,9 +838,9 @@ format_proc_cpuinfo (char *destbuf, size_t maxsize)
 		print (" psn");
 	      if (features1 & (1 << 19))
 		print (" clfl");
-	      if (features1 & (1 << 21))
+	      if (is_intel && features1 & (1 << 21))
 		print (" dtes");
-	      if (features1 & (1 << 22))
+	      if (is_intel && features1 & (1 << 22))
 		print (" acpi");
 	      if (features1 & (1 << 23))
 		print (" mmx");
@@ -843,28 +848,59 @@ format_proc_cpuinfo (char *destbuf, size_t maxsize)
 		print (" fxsr");
 	      if (features1 & (1 << 25))
 		print (" sse");
-	      if (features1 & (1 << 26))
-		print (" sse2");
-	      if (features1 & (1 << 27))
-		print (" ss");
-	      if (features1 & (1 << 28))
-		print (" htt");
-	      if (features1 & (1 << 29))
-		print (" tmi");
-	      if (features1 & (1 << 30))
-		print (" ia-64");
-	      if (features1 & (1 << 31))
-		print (" pbe");
-	      if (features2 & (1 << 0))
-		print (" sse3");
-	      if (features2 & (1 << 3))
-		print (" mon");
-	      if (features2 & (1 << 4))
-		print (" dscpl");
-	      if (features2 & (1 << 8))
-		print (" tm2");
-	      if (features2 & (1 << 10))
-		print (" cid");
+	      if (is_intel)
+		{
+		  if (features1 & (1 << 26))
+		    print (" sse2");
+		  if (features1 & (1 << 27))
+		    print (" ss");
+		  if (features1 & (1 << 28))
+		    print (" htt");
+		  if (features1 & (1 << 29))
+		    print (" tmi");
+		  if (features1 & (1 << 30))
+		    print (" ia-64");
+		  if (features1 & (1 << 31))
+		    print (" pbe");
+
+		  if (features2 & (1 << 0))
+		    print (" pni");
+		  if (features2 & (1 << 3))
+		    print (" monitor");
+		  if (features2 & (1 << 4))
+		    print (" ds_cpl");
+		  if (features2 & (1 << 7))
+		    print (" tm2");
+		  if (features2 & (1 << 8))
+		    print (" est");
+		 if (features2 & (1 << 10))
+		    print (" cid");
+		}
+
+	      if (is_amd && maxe >= 0x80000001)
+		{
+		  // uses AMD extended calls to check
+		  // for 3dnow and 3dnow extended support
+		  // (source: AMD Athlon Processor Recognition Application Note)
+
+		  if (maxe >= 0x80000001)  // has basic capabilities
+		    {
+		      cpuid (&unused, &unused, &unused, &features2, 0x80000001);
+
+		      if (features2 & (1 << 11))
+			print (" syscall");
+		      if (features2 & (1 << 19))
+			print (" mp");
+		      if (features2 & (1 << 22))
+			print (" mmxext");
+		      if (features2 & (1 << 29))
+			print (" lm");
+		      if (features2 & (1 << 30)) // 31th bit is on
+			print (" 3dnowext");
+		      if (features2 & (1 << 31)) // 32th bit (highest) is on
+			print (" 3dnow");
+		    }
+		}
 	    }
 	  else if (wincap.is_winnt ())
 	    {
@@ -964,7 +1000,7 @@ format_proc_partitions (char *destbuf, size_t maxsize)
 			continue;
 		      char devname[16];
 		      __small_sprintf (devname, "/dev/sd%c%d",
-		      				drive_number + 'a',
+						drive_number + 'a',
 						partition + 1);
 		      device dev;
 		      dev.parse (devname);
