@@ -25,7 +25,7 @@ int
 fhandler_dev_zero::open (int flags, mode_t)
 {
   set_flags ((flags & ~O_TEXT) | O_BINARY);
-  set_nohandle (true);
+  nohandle (true);
   set_open_status ();
   return 1;
 }
