@@ -80,6 +80,9 @@
 #include <unistd.h>
 #include <windows.h>
 
+#include "perprocess.h"
+#include "cygwin/version.h"
+
 #ifdef __weak_alias
 #ifdef __LIBC12_SOURCE__
 __weak_alias(glob,_glob);
@@ -94,11 +97,7 @@ __weak_alias(__globfree13,___globfree13);
 #ifdef __LIBC12_SOURCE__
 #define	STAT	stat12
 #else
-#if defined (__INSIDE_CYGWIN__)
 #define STAT	__stat64
-#else
-#define	STAT	stat
-#endif
 #endif
 
 #define	DOLLAR		'$'
@@ -386,7 +385,7 @@ globtilde(pattern, patbuf, pglob)
 		 * first and then trying the password file
 		 */
 		if ((h = getenv("HOME")) == NULL) {
-			if ((pwd = getpwuid(getuid())) == NULL)
+			if ((pwd = getpwuid32(getuid32())) == NULL)
 				return pattern;
 			else
 				h = pwd->pw_dir;
@@ -831,16 +830,6 @@ g_lstat(fn, sb, pglob)
 	struct STAT *sb;
 	glob_t *pglob;
 {
-	/* FIXME: This only works as long as the application uses the old
-	   struct stat with 32 bit off_t types!!!
-
-	   As soon as we switch over to 64 bit, we have to decide by
-	   the applications API minor version number, whether to use
-	   a pointer to a __stat64 or a _stat32 struct to the
-	   pglob->gl_lstat function. */
-#ifdef __CYGWIN_USE_BIG_TYPES__
-#error FIXME check apps API minor and use correct struct stat
-#endif
 	char buf[MAXPATHLEN];
 
 	g_Ctoc(fn, buf);
@@ -848,15 +837,13 @@ g_lstat(fn, sb, pglob)
 		struct __stat32 lsb;
 		int ret;
 
-		if (!(ret = (*pglob->gl_lstat)(buf, &lsb)))
+		if (CYGWIN_VERSION_CHECK_FOR_USING_BIG_TYPES)
+		  ret = (*pglob->gl_lstat)(buf, &sb);
+		else if (!(ret = (*pglob->gl_lstat)(buf, &lsb)))
 			stat32_to_STAT (&lsb, sb);
 		return ret;
 	}
-#ifdef __INSIDE_CYGWIN__
 	return(lstat64(buf, sb));
-#else
-	return(lstat(buf, sb));
-#endif
 }
 
 static int
@@ -865,16 +852,6 @@ g_stat(fn, sb, pglob)
 	struct STAT *sb;
 	glob_t *pglob;
 {
-	/* FIXME: This only works as long as the application uses the old
-	   struct stat with 32 bit off_t types!!!
-
-	   As soon as we switch over to 64 bit, we have to decide by
-	   the applications API minor version number, whether to use
-	   a pointer to a __stat64 or a _stat32 struct to the
-	   pglob->gl_stat function. */
-#ifdef __CYGWIN_USE_BIG_TYPES__
-#error FIXME check apps API minor and use correct struct stat
-#endif
 	char buf[MAXPATHLEN];
 
 	g_Ctoc(fn, buf);
@@ -882,15 +859,13 @@ g_stat(fn, sb, pglob)
 		struct __stat32 lsb;
 		int ret;
 
+		if (CYGWIN_VERSION_CHECK_FOR_USING_BIG_TYPES)
+		  ret = (*pglob->gl_stat)(buf, &sb);
 		if (!(ret = (*pglob->gl_stat)(buf, &lsb)))
 			stat32_to_STAT (&lsb, sb);
 		return ret;
 	}
-#ifdef __INSIDE_CYGWIN__
 	return(stat64(buf, sb));
-#else
-	return(stat(buf, sb));
-#endif
 }
 
 static Char *
