@@ -967,6 +967,7 @@ cygwin_gethostbyname (const char *name)
 				     is_hostent);
   if (!_my_tls.locals.hostent_buf)
     {
+      debug_printf ("name %s", name);
       set_winsock_errno ();
       set_host_errno ();
     }
@@ -1169,7 +1170,7 @@ cygwin_getpeername (int fd, struct sockaddr *name, int *len)
   else
     res = fh->getpeername (name, len);
 
-  syscall_printf ("%d = getpeername %d", res, (fh ? fh->get_socket () : -1));
+  syscall_printf ("%d = getpeername (%d) %d", res, fd, (fh ? fh->get_socket () : -1));
   return res;
 }
 
@@ -1953,7 +1954,7 @@ cygwin_rresvport (int *port)
       cygheap_fdnew res_fd;
 
       if (res_fd >= 0 && fdsock (res_fd, tcp_dev, res))
-        res = res_fd;
+	res = res_fd;
       else
 	res = -1;
     }
