@@ -839,9 +839,8 @@ out:
     {
       if (fs.update (path))
 	{
-	  set_isdisk ();
 	  debug_printf ("this->path(%s), has_acls(%d)", path, fs.has_acls ());
-	  if (fs.has_acls () && allow_ntsec && wincap.has_security ())
+	  if (fs.has_acls () && allow_ntsec)
 	    set_exec (0);  /* We really don't know if this is executable or not here
 			      but set it to not executable since it will be figured out
 			      later by anything which cares about this. */
@@ -978,29 +977,17 @@ normalize_win32_path (const char *src, char *dst, char **tail)
 	  src += 2;
 	}
     }
-  else if (strchr (src, ':') == NULL && *src != '/')
+  else if (!isdrive(src) && *src != '/')
     {
-      if (!cygheap->cwd.get (dst, 0))
-	return get_errno ();
       if (beg_src_slash)
+	dst += cygheap->cwd.get_drive (dst);
+      else if (!cygheap->cwd.get (dst, 0))
+	return get_errno ();
+      else
 	{
-	  if (dst[1] == ':')
-	    dst[2] = '\0';
-	  else if (is_unc_share (dst))
-	    {
-	      char *p = strpbrk (dst + 2, "\\/");
-	      if (p && (p = strpbrk (p + 1, "\\/")))
-		  *p = '\0';
-	    }
+	  dst += strlen (dst);
+	  *dst++ = '\\';
 	}
-      if (strlen (dst) + 1 + strlen (src) >= CYG_MAX_PATH)
-	{
-	  debug_printf ("ENAMETOOLONG = normalize_win32_path (%s)", src);
-	  return ENAMETOOLONG;
-	}
-      dst += strlen (dst);
-      if (!beg_src_slash)
-	*dst++ = '\\';
     }
 
   while (*src)
@@ -1035,6 +1022,7 @@ normalize_win32_path (const char *src, char *dst, char **tail)
 	      int n = strspn (src, ".");
 	      if (!src[n] || isdirsep (src[n])) /* just dots... */
 		return ENOENT;
+	      *dst++ = *src++;
 	    }
 	}
       /* Otherwise, add char to result.  */
@@ -1466,8 +1454,7 @@ mount_info::conv_to_win32_path (const char *src_path, char *dst, device& dev,
       int n = mount_table->cygdrive_len - 1;
       int unit;
 
-      if (!src_path[n] ||
-	  (src_path[n] == '/' && src_path[n + 1] == '.' && !src_path[n + 2]))
+      if (!src_path[n])
 	{
 	  unit = 0;
 	  dst[0] = '\0';
@@ -1521,8 +1508,12 @@ mount_info::conv_to_win32_path (const char *src_path, char *dst, device& dev,
       chroot_ok = true;
     }
   else
-    backslashify (src_path, dst, 0);
-
+    {
+      int offset = 0;
+      if (src_path[1] != '/' && src_path[1] != ':')
+	offset = cygheap->cwd.get_drive (dst);
+      backslashify (src_path, dst + offset, 0);
+    }
  out:
   MALLOC_CHECK;
   if (chroot_ok || cygheap->root.ischroot_native (dst))
@@ -1811,11 +1802,13 @@ mount_info::from_registry ()
   read_mounts (r);
 
   /* Then read mounts from system-wide mount table. */
+  cygheap->user.deimpersonate ();
   reg_key r1 (HKEY_LOCAL_MACHINE, KEY_READ, "SOFTWARE",
 	      CYGWIN_INFO_CYGNUS_REGISTRY_NAME, CYGWIN_REGNAME,
 	      CYGWIN_INFO_CYGWIN_MOUNT_REGISTRY_NAME,
 	      NULL);
   read_mounts (r1);
+  cygheap->user.reimpersonate ();
 }
 
 /* add_reg_mount: Add mount item to registry.  Return zero on success,
@@ -1931,21 +1924,21 @@ mount_info::read_cygdrive_info_from_registry ()
 {
   /* reg_key for user path prefix in HKEY_CURRENT_USER. */
   reg_key r;
-
+  /* First read cygdrive from user's registry. */
   if (r.get_string (CYGWIN_INFO_CYGDRIVE_PREFIX, cygdrive, sizeof (cygdrive), "") != 0)
     {
-      /* Didn't find the user path prefix so check the system path prefix. */
-
-      /* reg_key for system path prefix in HKEY_LOCAL_MACHINE.  */
+      /* Then read cygdrive from system-wide registry. */
+      cygheap->user.deimpersonate ();
       reg_key r2 (HKEY_LOCAL_MACHINE, KEY_READ, "SOFTWARE",
 		 CYGWIN_INFO_CYGNUS_REGISTRY_NAME, CYGWIN_REGNAME,
 		 CYGWIN_INFO_CYGWIN_MOUNT_REGISTRY_NAME,
 		 NULL);
+      cygheap->user.reimpersonate ();
 
       if (r2.get_string (CYGWIN_INFO_CYGDRIVE_PREFIX, cygdrive,
 	  sizeof (cygdrive), ""))
 	strcpy (cygdrive, CYGWIN_INFO_CYGDRIVE_DEFAULT_PREFIX);
-      cygdrive_flags = r2.get_int (CYGWIN_INFO_CYGDRIVE_FLAGS, MOUNT_CYGDRIVE);
+      cygdrive_flags = r2.get_int (CYGWIN_INFO_CYGDRIVE_FLAGS, MOUNT_CYGDRIVE | MOUNT_BINARY);
       slashify (cygdrive, cygdrive, 1);
       cygdrive_len = strlen (cygdrive);
     }
@@ -1953,7 +1946,7 @@ mount_info::read_cygdrive_info_from_registry ()
     {
       /* Fetch user cygdrive_flags from registry; returns MOUNT_CYGDRIVE on
 	 error. */
-      cygdrive_flags = r.get_int (CYGWIN_INFO_CYGDRIVE_FLAGS, MOUNT_CYGDRIVE);
+      cygdrive_flags = r.get_int (CYGWIN_INFO_CYGDRIVE_FLAGS, MOUNT_CYGDRIVE | MOUNT_BINARY);
       slashify (cygdrive, cygdrive, 1);
       cygdrive_len = strlen (cygdrive);
     }
@@ -2060,7 +2053,7 @@ mount_info::get_cygdrive_info (char *user, char *system, char* user_flags,
   /* Get the user flags, if appropriate */
   if (res == ERROR_SUCCESS)
     {
-      int flags = r.get_int (CYGWIN_INFO_CYGDRIVE_FLAGS, MOUNT_CYGDRIVE);
+      int flags = r.get_int (CYGWIN_INFO_CYGDRIVE_FLAGS, MOUNT_CYGDRIVE | MOUNT_BINARY);
       strcpy (user_flags, (flags & MOUNT_BINARY) ? "binmode" : "textmode");
     }
 
@@ -2074,7 +2067,7 @@ mount_info::get_cygdrive_info (char *user, char *system, char* user_flags,
   /* Get the system flags, if appropriate */
   if (res2 == ERROR_SUCCESS)
     {
-      int flags = r2.get_int (CYGWIN_INFO_CYGDRIVE_FLAGS, MOUNT_CYGDRIVE);
+      int flags = r2.get_int (CYGWIN_INFO_CYGDRIVE_FLAGS, MOUNT_CYGDRIVE | MOUNT_BINARY);
       strcpy (system_flags, (flags & MOUNT_BINARY) ? "binmode" : "textmode");
     }
 
@@ -2197,7 +2190,7 @@ mount_info::add_item (const char *native, const char *posix, unsigned mountflags
     posixerr = normalize_posix_path (posix, posixtmp, &posixtail);
 
   debug_printf ("%s[%s], %s[%s], %p",
-                native, nativeerr ? error : nativetmp,
+		native, nativeerr ? error : nativetmp,
 		posix, posixerr ? error : posixtmp, mountflags);
 
   if (nativeerr || posixerr)
@@ -3257,15 +3250,13 @@ hash_path_name (__ino64_t hash, const char *name)
     }
 
 hashit:
-  /* Build up hash.  Ignore single trailing slash or \a\b\ != \a\b or
-     \a\b\.  but allow a single \ if that's all there is. */
+  /* Build up hash. Name is already normalized */
   do
     {
       int ch = cyg_tolower (*name);
       hash = ch + (hash << 6) + (hash << 16) - hash;
     }
-  while (*++name != '\0' &&
-	 !(*name == '\\' && (!name[1] || (name[1] == '.' && !name[2]))));
+  while (*++name != '\0');
   return hash;
 }
 
@@ -3705,6 +3696,17 @@ cwdstuff::set (const char *win32_cwd, const char *posix_cwd, bool doit)
       win32 = (char *) crealloc (win32, strlen (win32_cwd) + 1);
       strcpy (win32, win32_cwd);
     }
+  if (win32[1] == ':')
+    drive_length = 2;
+  else if (win32[1] == '\\')
+    {
+      char * ptr = strechr (win32 + 2, '\\');
+      if (*ptr)
+	ptr = strechr (ptr + 1, '\\');
+      drive_length = ptr - win32;
+    }
+  else
+    drive_length = 0;
 
   if (!posix_cwd)
     {
