@@ -69,7 +69,7 @@ extern "C" void
 cygwin_set_impersonation_token (const HANDLE hToken)
 {
   debug_printf ("set_impersonation_token (%d)", hToken);
-  cygheap->user.external_token = hToken;
+  cygheap->user.external_token = hToken == INVALID_HANDLE_VALUE ? NO_IMPERSONATION : hToken;
   return;
 }
 
@@ -163,8 +163,8 @@ str2buf2uni (UNICODE_STRING &tgt, WCHAR *buf, const char *srcstr)
 {
   tgt.Buffer = (PWCHAR) buf;
   tgt.MaximumLength = (strlen (srcstr) + 1) * sizeof (WCHAR);
-  tgt.Length = sys_mbstowcs (buf, srcstr, tgt.MaximumLength / sizeof (WCHAR)) 
-               * sizeof (WCHAR);
+  tgt.Length = sys_mbstowcs (buf, srcstr, tgt.MaximumLength / sizeof (WCHAR))
+	       * sizeof (WCHAR);
   if (tgt.Length)
     tgt.Length -= sizeof (WCHAR);
 }
@@ -173,7 +173,7 @@ void
 str2uni_cat (UNICODE_STRING &tgt, const char *srcstr)
 {
   int len = sys_mbstowcs (tgt.Buffer + tgt.Length / sizeof (WCHAR), srcstr,
-		          (tgt.MaximumLength - tgt.Length) / sizeof (WCHAR));
+			  (tgt.MaximumLength - tgt.Length) / sizeof (WCHAR));
   if (len)
     tgt.Length += (len - 1) * sizeof (WCHAR);
   else
@@ -1382,7 +1382,7 @@ int
 get_object_attribute (HANDLE handle, SE_OBJECT_TYPE object_type,
 		      mode_t *attribute, __uid32_t *uidret, __gid32_t *gidret)
 {
-  if (allow_ntsec && wincap.has_security ())
+  if (allow_ntsec)
     {
       get_nt_object_attribute (handle, object_type, attribute, uidret, gidret);
       return 0;
@@ -1398,7 +1398,7 @@ get_file_attribute (int use_ntsec, HANDLE handle, const char *file,
   int res;
   syscall_printf ("file: %s", file);
 
-  if (use_ntsec && allow_ntsec && wincap.has_security ())
+  if (use_ntsec && allow_ntsec)
     {
       if (!handle || get_nt_object_attribute (handle, SE_FILE_OBJECT,
 					      attribute, uidret, gidret))
