@@ -11,11 +11,8 @@ This software is a copyrighted work licensed under the terms of the
 Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
-/*
- * The following line means that the BSD socket
- * definitions for fd_set, FD_ISSET etc. are used in this
- * file.
- */
+/* The following line means that the BSD socket definitions for
+   fd_set, FD_ISSET etc. are used in this file.  */
 
 #define  __INSIDE_CYGWIN_NET__
 
@@ -43,6 +40,8 @@ details. */
 #include "tty.h"
 #include "cygthread.h"
 #include "ntdll.h"
+#include "cygtls.h"
+#include <asm/byteorder.h>
 
 /*
  * All these defines below should be in sys/types.h
@@ -507,63 +506,63 @@ out:
   if (s->write_selected)
     {
       if (s->write_ready)
-        {
-          select_printf ("%s, already ready for write", fh->get_name ());
-          gotone++;
-        }
+	{
+	  select_printf ("%s, already ready for write", fh->get_name ());
+	  gotone++;
+	}
       /* Do we need to do anything about SIGTTOU here? */
       else if (fh->get_device () == FH_PIPER)
 	select_printf ("%s, select for write on read end of pipe",
 		       fh->get_name ());
       else
-        {
-          /* We don't worry about the guard mutex, because that only applies
-             when from_select is false, and peek_pipe is never called that
-             way for writes.  */
+	{
+	  /* We don't worry about the guard mutex, because that only applies
+	     when from_select is false, and peek_pipe is never called that
+	     way for writes.  */
 
-          IO_STATUS_BLOCK iosb = {0};
-          FILE_PIPE_LOCAL_INFORMATION fpli = {0};
+	  IO_STATUS_BLOCK iosb = {0};
+	  FILE_PIPE_LOCAL_INFORMATION fpli = {0};
 
-          if (NtQueryInformationFile (h,
-                                      &iosb,
-                                      &fpli,
-                                      sizeof (fpli),
-                                      FilePipeLocalInformation))
-            {
-              /* If NtQueryInformationFile fails, optimistically assume the
-                 pipe is writable.  This could happen on Win9x, because
-                 NtQueryInformationFile is not available, or if we somehow
-                 inherit a pipe that doesn't permit FILE_READ_ATTRIBUTES
-                 access on the write end.  */
-              select_printf ("%s, NtQueryInformationFile failed",
-                             fh->get_name ());
-              gotone += s->write_ready = true;
-            }
-          /* Ensure that enough space is available for atomic writes,
-             as required by POSIX.  Subsequent writes with size > PIPE_BUF
-             can still block, but most (all?) UNIX variants seem to work
-             this way (e.g., BSD, Linux, Solaris).  */
-          else if (fpli.WriteQuotaAvailable >= PIPE_BUF)
-            {
-              select_printf ("%s, ready for write: size %lu, avail %lu",
-                             fh->get_name (),
-                             fpli.OutboundQuota,
-                             fpli.WriteQuotaAvailable);
-              gotone += s->write_ready = true;
-            }
-          /* If we somehow inherit a tiny pipe (size < PIPE_BUF), then consider
-             the pipe writable only if it is completely empty, to minimize the
-             probability that a subsequent write will block.  */
-          else if (fpli.OutboundQuota < PIPE_BUF &&
-                   fpli.WriteQuotaAvailable == fpli.OutboundQuota)
-            {
-              select_printf ("%s, tiny pipe: size %lu, avail %lu",
-                             fh->get_name (),
-                             fpli.OutboundQuota,
-                             fpli.WriteQuotaAvailable);
-              gotone += s->write_ready = true;
-            }
-        }
+	  if (NtQueryInformationFile (h,
+				      &iosb,
+				      &fpli,
+				      sizeof (fpli),
+				      FilePipeLocalInformation))
+	    {
+	      /* If NtQueryInformationFile fails, optimistically assume the
+		 pipe is writable.  This could happen on Win9x, because
+		 NtQueryInformationFile is not available, or if we somehow
+		 inherit a pipe that doesn't permit FILE_READ_ATTRIBUTES
+		 access on the write end.  */
+	      select_printf ("%s, NtQueryInformationFile failed",
+			     fh->get_name ());
+	      gotone += s->write_ready = true;
+	    }
+	  /* Ensure that enough space is available for atomic writes,
+	     as required by POSIX.  Subsequent writes with size > PIPE_BUF
+	     can still block, but most (all?) UNIX variants seem to work
+	     this way (e.g., BSD, Linux, Solaris).  */
+	  else if (fpli.WriteQuotaAvailable >= PIPE_BUF)
+	    {
+	      select_printf ("%s, ready for write: size %lu, avail %lu",
+			     fh->get_name (),
+			     fpli.OutboundQuota,
+			     fpli.WriteQuotaAvailable);
+	      gotone += s->write_ready = true;
+	    }
+	  /* If we somehow inherit a tiny pipe (size < PIPE_BUF), then consider
+	     the pipe writable only if it is completely empty, to minimize the
+	     probability that a subsequent write will block.  */
+	  else if (fpli.OutboundQuota < PIPE_BUF &&
+		   fpli.WriteQuotaAvailable == fpli.OutboundQuota)
+	    {
+	      select_printf ("%s, tiny pipe: size %lu, avail %lu",
+			     fh->get_name (),
+			     fpli.OutboundQuota,
+			     fpli.WriteQuotaAvailable);
+	      gotone += s->write_ready = true;
+	    }
+	}
     }
 
   return gotone;
@@ -1211,7 +1210,6 @@ struct socketinf
     cygthread *thread;
     winsock_fd_set readfds, writefds, exceptfds;
     SOCKET exitsock;
-    struct sockaddr_in sin;
     select_record *start;
   };
 
@@ -1306,12 +1304,9 @@ thread_socket (void *arg)
 
   if (WINSOCK_FD_ISSET (si->exitsock, &si->readfds))
     select_printf ("saw exitsock read");
-
   return 0;
 }
 
-extern "C" unsigned long htonl (unsigned long);
-
 static int
 start_thread_socket (select_record *me, select_stuff *stuff)
 {
@@ -1350,41 +1345,40 @@ start_thread_socket (select_record *me, select_stuff *stuff)
 	  }
       }
 
-  if ((si->exitsock = socket (PF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET)
+  if (_my_tls.locals.exitsock != INVALID_SOCKET)
+    si->exitsock = _my_tls.locals.exitsock;
+  else 
     {
-      set_winsock_errno ();
-      select_printf ("cannot create socket, %E");
-      return -1;
-    }
-  /* Allow rapid reuse of the port. */
-  int tmp = 1;
-  (void) setsockopt (si->exitsock, SOL_SOCKET, SO_REUSEADDR, (char *) &tmp, sizeof (tmp));
+      si->exitsock = _my_tls.locals.exitsock = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+      if (_my_tls.locals.exitsock == INVALID_SOCKET)
+	{
+	  set_winsock_errno ();
+	  select_printf ("cannot create socket, %E");
+	  return -1;
+	}
+      int sin_len = sizeof (_my_tls.locals.exitsock_sin);
+      memset (&_my_tls.locals.exitsock_sin, 0, sin_len);
+      _my_tls.locals.exitsock_sin.sin_family = AF_INET;
+      _my_tls.locals.exitsock_sin.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
+      if (bind (si->exitsock, (struct sockaddr *) &_my_tls.locals.exitsock_sin, sin_len) < 0)
+	{
+	  select_printf ("cannot bind socket %p, %E", si->exitsock);
+	  goto err;
+	}
 
-  int sin_len = sizeof (si->sin);
-  memset (&si->sin, 0, sizeof (si->sin));
-  si->sin.sin_family = AF_INET;
-  si->sin.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
-  if (bind (si->exitsock, (struct sockaddr *) &si->sin, sizeof (si->sin)) < 0)
-    {
-      select_printf ("cannot bind socket, %E");
-      goto err;
-    }
-
-  if (getsockname (si->exitsock, (struct sockaddr *) &si->sin, &sin_len) < 0)
-    {
-      select_printf ("getsockname error");
-      goto err;
-    }
-
-  if (listen (si->exitsock, 1))
-    {
-      select_printf ("listen failed, %E");
-      goto err;
+      if (getsockname (si->exitsock, (struct sockaddr *) &_my_tls.locals.exitsock_sin, &sin_len) < 0)
+	{
+	  select_printf ("getsockname error");
+	  goto err;
+	}
+      if (winsock2_active && wincap.has_set_handle_information ())
+	SetHandleInformation ((HANDLE) si->exitsock, HANDLE_FLAG_INHERIT, 0);
+      /* else
+	   too bad? */
     }
 
   select_printf ("exitsock %p", si->exitsock);
   WINSOCK_FD_SET ((HANDLE) si->exitsock, &si->readfds);
-  WINSOCK_FD_SET ((HANDLE) si->exitsock, &si->exceptfds);
   stuff->device_specific_socket = (void *) si;
   si->start = &stuff->start;
   select_printf ("stuff_start %p", &stuff->start);
@@ -1405,30 +1399,17 @@ socket_cleanup (select_record *, select_stuff *stuff)
   select_printf ("si %p si->thread %p", si, si ? si->thread : NULL);
   if (si && si->thread)
     {
-      select_printf ("connection to si->exitsock %p", si->exitsock);
-      SOCKET s = socket (AF_INET, SOCK_STREAM, 0);
-
-      /* Set LINGER with 0 timeout for hard close */
-      struct linger tmp = {1, 0}; /* On, 0 delay */
-      (void) setsockopt (s, SOL_SOCKET, SO_LINGER, (char *)&tmp, sizeof (tmp));
-      (void) setsockopt (si->exitsock, SOL_SOCKET, SO_LINGER, (char *)&tmp, sizeof (tmp));
-
-      /* Connecting to si->exitsock will cause any executing select to wake
-	 up.  When this happens then the exitsock condition will cause the
-	 thread to terminate. */
-      if (connect (s, (struct sockaddr *) &si->sin, sizeof (si->sin)) < 0)
-	{
-	  set_winsock_errno ();
-	  select_printf ("connect failed");
-	  /* FIXME: now what? */
-	}
-      shutdown (s, SD_BOTH);
-      closesocket (s);
-
+      char buf[] = "";
+      int res = sendto (_my_tls.locals.exitsock, buf, 1, 0,
+			(sockaddr *) &_my_tls.locals.exitsock_sin,
+			sizeof (_my_tls.locals.exitsock_sin));
+      select_printf ("sent a byte to exitsock %p, res %d", _my_tls.locals.exitsock, res);
       /* Wait for thread to go away */
       si->thread->detach ();
-      shutdown (si->exitsock, SD_BOTH);
-      closesocket (si->exitsock);
+      /* empty the socket */
+      select_printf ("reading a byte from exitsock %p", si->exitsock);
+      res = recv (si->exitsock, buf, 1, 0);
+      select_printf ("recv returned %d", res);
       stuff->device_specific_socket = NULL;
       delete si;
     }
