@@ -24,7 +24,9 @@ enum
   __SIGCOMMUNE	    = -(NSIG + 3),
   __SIGPENDING	    = -(NSIG + 4),
   __SIGDELETE	    = -(NSIG + 5),
-  __SIGFLUSHFAST    = -(NSIG + 6)
+  __SIGFLUSHFAST    = -(NSIG + 6),
+  __SIGHOLD	    = -(NSIG + 7),
+  __SIGNOHOLD	    = -(NSIG + 8)
 };
 #endif
 
