@@ -175,8 +175,24 @@ unlink (const char *ourname)
 					| FILE_ATTRIBUTE_SYSTEM));
     }
   /* Attempt to use "delete on close" semantics to handle removing
-     a file which may be open. */
-  if (wincap.has_delete_on_close ())
+     a file which may be open.
+
+     CV 2004-09-17: Not if the file is on a remote share.  If two processes
+     have open handles on a file and one of them calls unlink, then it
+     happens that the file is remove from the remote share even though the
+     other process still has an open handle.  This other process than gets
+     Win32 error 59, ERROR_UNEXP_NET_ERR when trying to access the file. 
+
+     For some reason, that does not happen when using DeleteFile, which
+     nicely succeeds but still, the file is available for the other process. 
+     To reproduce, mount /tmp on a remote share and call
+     
+       bash -c "cat << EOF"
+
+     Microsoft KB 837665 describes this problem as a bug in 2K3, but I have
+     reproduced it on shares on Samba 2.2.8, Samba 3.0.2, NT4SP6, XP64SP1 and
+     2K3 and in all cases, DeleteFile works, "delete on close" does not. */
+  if (!win32_name.isremote () && wincap.has_delete_on_close ())
     {
       HANDLE h;
       h = CreateFile (win32_name, 0, FILE_SHARE_READ, &sec_none_nih,
@@ -370,8 +386,7 @@ read (int fd, void *ptr, size_t len)
   return readv (fd, &iov, 1);
 }
 
-extern "C" ssize_t _read (int, void *, size_t)
-  __attribute__ ((alias ("read")));
+EXPORT_ALIAS (read, _read)
 
 extern "C" ssize_t
 write (int fd, const void *ptr, size_t len)
@@ -385,8 +400,7 @@ write (int fd, const void *ptr, size_t len)
   return writev (fd, &iov, 1);
 }
 
-extern "C" ssize_t _write (int fd, const void *ptr, size_t len)
-  __attribute__ ((alias ("write")));
+EXPORT_ALIAS (write, _write)
 
 extern "C" ssize_t
 readv (int fd, const struct iovec *const iov, const int iovcnt)
@@ -572,11 +586,8 @@ open (const char *unix_path, int flags, ...)
   return res;
 }
 
-extern "C" int _open (const char *, int flags, ...)
-  __attribute__ ((alias ("open")));
-
-extern "C" int _open64 (const char *, int flags, ...)
-  __attribute__ ((alias ("open")));
+EXPORT_ALIAS (open, _open )
+EXPORT_ALIAS (open, _open64 )
 
 extern "C" _off64_t
 lseek64 (int fd, _off64_t pos, int dir)
@@ -601,8 +612,7 @@ lseek64 (int fd, _off64_t pos, int dir)
   return res;
 }
 
-extern "C" int _lseek64 (int fd, _off64_t pos, int dir)
-  __attribute__ ((alias ("lseek64")));
+EXPORT_ALIAS (lseek64, _lseek64)
 
 extern "C" _off_t
 lseek (int fd, _off_t pos, int dir)
@@ -610,8 +620,7 @@ lseek (int fd, _off_t pos, int dir)
   return lseek64 (fd, (_off64_t) pos, dir);
 }
 
-extern "C" _off_t _lseek (int, _off_t, int)
-  __attribute__ ((alias ("lseek")));
+EXPORT_ALIAS (lseek, _lseek)
 
 extern "C" int
 close (int fd)
@@ -635,7 +644,7 @@ close (int fd)
   return res;
 }
 
-extern "C" int _close (int) __attribute__ ((alias ("close")));
+EXPORT_ALIAS (close, _close)
 
 extern "C" int
 isatty (int fd)
@@ -719,7 +728,7 @@ link (const char *a, const char *b)
 	  goto docopy;
 	}
       if (GetLastError () != ERROR_PROC_NOT_FOUND)
-        {
+	{
 	  syscall_printf ("CreateHardLinkA failed");
 	  __seterrno ();
 	  goto done;
@@ -811,7 +820,7 @@ link (const char *a, const char *b)
       CloseHandle (hFileSource);
 
       if (!bSuccess)
-        {
+	{
 	  /* Only copy file if FS doesn't support hard links */
 	  if (write_err == ERROR_INVALID_FUNCTION)
 	    {
@@ -2023,17 +2032,17 @@ seteuid32 (__uid32_t uid)
   if (verify_token (ptok, usersid, groups))
     new_token = ptok;
   /* Verify if the external token is suitable */
-  else if (cygheap->user.external_token != INVALID_HANDLE_VALUE
+  else if (cygheap->user.external_token != NO_IMPERSONATION
 	   && verify_token (cygheap->user.external_token, usersid, groups))
     new_token = cygheap->user.external_token;
   /* Verify if the current token (internal or former external) is suitable */
-  else if (cygheap->user.current_token != INVALID_HANDLE_VALUE
+  else if (cygheap->user.current_token != NO_IMPERSONATION
 	   && cygheap->user.current_token != cygheap->user.external_token
 	   && verify_token (cygheap->user.current_token, usersid, groups,
 			    &token_is_internal))
     new_token = cygheap->user.current_token;
   /* Verify if the internal token is suitable */
-  else if (cygheap->user.internal_token != INVALID_HANDLE_VALUE
+  else if (cygheap->user.internal_token != NO_IMPERSONATION
 	   && cygheap->user.internal_token != cygheap->user.current_token
 	   && verify_token (cygheap->user.internal_token, usersid, groups,
 			    &token_is_internal))
@@ -2065,10 +2074,11 @@ seteuid32 (__uid32_t uid)
 	    goto failed;
 	}
       /* Keep at most one internal token */
-      if (cygheap->user.internal_token != INVALID_HANDLE_VALUE)
+      if (cygheap->user.internal_token != NO_IMPERSONATION)
 	CloseHandle (cygheap->user.internal_token);
       cygheap->user.internal_token = new_token;
     }
+
   if (new_token != ptok)
     {
       /* Avoid having HKCU use default user */
@@ -2094,7 +2104,7 @@ seteuid32 (__uid32_t uid)
   CloseHandle (ptok);
   issamesid = (usersid == cygheap->user.sid ());
   cygheap->user.set_sid (usersid);
-  cygheap->user.current_token = new_token == ptok ? INVALID_HANDLE_VALUE
+  cygheap->user.current_token = new_token == ptok ? NO_IMPERSONATION
 						  : new_token;
   if (!issamesid) /* MS KB 199190 */
     RegCloseKey (HKEY_CURRENT_USER);
@@ -2482,55 +2492,6 @@ updwtmp (const char *wtmp_file, const struct utmp *ut)
     }
 }
 
-extern "C" void
-logwtmp (const char *line, const char *user, const char *host)
-{
-  struct utmp ut;
-  memset (&ut, 0, sizeof ut);
-  ut.ut_type = USER_PROCESS;
-  ut.ut_pid = getpid ();
-  if (line)
-    strncpy (ut.ut_line, line, sizeof ut.ut_line);
-  time (&ut.ut_time);
-  if (user)
-    strncpy (ut.ut_user, user, sizeof ut.ut_user);
-  if (host)
-    strncpy (ut.ut_host, host, sizeof ut.ut_host);
-  updwtmp (_PATH_WTMP, &ut);
-}
-
-extern "C" void
-login (struct utmp *ut)
-{
-  pututline (ut);
-  endutent ();
-  updwtmp (_PATH_WTMP, ut);
-}
-
-extern "C" int
-logout (char *line)
-{
-  struct utmp ut_buf, *ut;
-
-  memset (&ut_buf, 0, sizeof ut_buf);
-  strncpy (ut_buf.ut_line, line, sizeof ut_buf.ut_line);
-  setutent ();
-  ut = getutline (&ut_buf);
-
-  if (ut)
-    {
-      ut->ut_type = DEAD_PROCESS;
-      memset (ut->ut_user, 0, sizeof ut->ut_user);
-      memset (ut->ut_host, 0, sizeof ut->ut_host);
-      time (&ut->ut_time);
-      debug_printf ("set logout time for %s", line);
-      pututline (ut);
-      endutent ();
-      return 1;
-    }
-  return 0;
-}
-
 static int utmp_fd = -1;
 static bool utmp_readonly = false;
 static char *utmp_file = (char *) _PATH_UTMP;
