@@ -160,9 +160,9 @@ uinfo_init ()
 
   cygheap->user.saved_uid = cygheap->user.real_uid = myself->uid;
   cygheap->user.saved_gid = cygheap->user.real_gid = myself->gid;
-  cygheap->user.external_token = INVALID_HANDLE_VALUE;
-  cygheap->user.internal_token = INVALID_HANDLE_VALUE;
-  cygheap->user.current_token = INVALID_HANDLE_VALUE;
+  cygheap->user.external_token = NO_IMPERSONATION;
+  cygheap->user.internal_token = NO_IMPERSONATION;
+  cygheap->user.current_token = NO_IMPERSONATION;
   cygheap->user.set_saved_sid ();	/* Update the original sid */
 }
 
@@ -449,6 +449,28 @@ cygheap_user::env_name (const char *name, size_t namelen)
   return pwinname;
 }
 
+const char *
+cygheap_user::env_systemroot (const char *name, size_t namelen)
+{
+  if (!psystemroot)
+    {
+      int size = GetWindowsDirectory (NULL, 0);
+      if (size > 0)
+        {
+	  psystemroot = (char *) cmalloc (HEAP_STR, ++size);
+	  size = GetWindowsDirectory (psystemroot, size);
+	  if (size <= 0)
+	    {
+	      cfree (psystemroot);
+	      psystemroot = NULL;
+	    }
+	}
+      if (size <= 0)
+        debug_printf ("GetWindowsDirectory(), %E");
+    }
+  return psystemroot;
+}
+
 char *
 pwdgrp::next_str (char c)
 {
@@ -511,7 +533,7 @@ pwdgrp::load (const char *posix_fname)
 
   paranoid_printf ("%s", posix_fname);
 
-  if (pc.error || !pc.exists () || !pc.isdisk () || pc.isdir ())
+  if (pc.error || !pc.exists () || pc.isdir ())
     {
       paranoid_printf ("strange path_conv problem");
       res = failed;
