@@ -1,6 +1,6 @@
 /* wait.cc: Posix wait routines.
 
-   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003 Red Hat, Inc.
+   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -15,6 +15,7 @@ details. */
 #include "sigproc.h"
 #include "perthread.h"
 #include "thread.h"
+#include "cygtls.h"
 
 /* This is called _wait and not wait because the real wait is defined
    in libc/syscalls/syswait.c.  It calls us.  */
@@ -46,40 +47,35 @@ extern "C" pid_t
 wait4 (int intpid, int *status, int options, struct rusage *r)
 {
   int res;
-  waitq *w;
   HANDLE waitfor;
-  bool sawsig;
+  waitq *w = &_my_tls.wq;
 
   pthread_testcancel ();
 
   while (1)
     {
       sig_dispatch_pending ();
-      sigframe thisframe (mainthread);
-      sawsig = 0;
       if (options & ~(WNOHANG | WUNTRACED))
 	{
 	  set_errno (EINVAL);
-	  return -1;
+	  res = -1;
+	  break;
 	}
 
       if (r)
 	memset (r, 0, sizeof (*r));
 
-      if ((w = (waitq *) waitq_storage.get ()) == NULL)
-	w = (waitq *) waitq_storage.create ();
-
       w->pid = intpid;
       w->options = options;
       w->rusage = r;
       sigproc_printf ("calling proc_subproc, pid %d, options %d",
-		     w->pid, w->options);
-      if (!proc_subproc (PROC_WAIT, (DWORD)w))
+		      w->pid, w->options);
+      if (!proc_subproc (PROC_WAIT, (DWORD) w))
 	{
 	  set_errno (ENOSYS);
 	  paranoid_printf ("proc_subproc returned 0");
 	  res = -1;
-	  goto done;
+	  break;
 	}
 
       if ((waitfor = w->ev) == NULL)
@@ -95,13 +91,14 @@ wait4 (int intpid, int *status, int options, struct rusage *r)
 	  /* found no children */
 	  set_errno (ECHILD);
 	  res = -1;
-	  goto done;
+	  break;
 	}
 
       if (w->status == -1)
 	{
+	  if (_my_tls.call_signal_handler ())
+	    continue;
 	  set_sig_errno (EINTR);
-	  sawsig = 1;
 	  res = -1;
 	}
       else if (res != WAIT_OBJECT_0)
@@ -113,16 +110,13 @@ wait4 (int intpid, int *status, int options, struct rusage *r)
 	}
       else if ((res = w->pid) != 0 && status)
 	*status = w->status;
-
-    done:
-      if (!sawsig || !thisframe.call_signal_handler ())
-	break;
+      break;
     }
 
   sigproc_printf ("intpid %d, status %p, w->status %d, options %d, res %d",
 		  intpid, status, w->status, options, res);
   w->status = -1;
   if (res < 0)
-    sigproc_printf ("*** errno = %d", get_errno ());
+    sigproc_printf ("*** errno %d", get_errno ());
   return res;
 }
