@@ -20,6 +20,8 @@
 #include "cygwin/include/sys/cygwin.h"
 #include "cygwin/include/mntent.h"
 
+#define alloca __builtin_alloca
+
 int verbose = 0;
 int registry = 0;
 int sysinfo = 0;
@@ -40,7 +42,7 @@ void dump_setup (int, char **, bool);
 void package_find (int, char **);
 void package_list (int, char **);
 
-static const char version[] = "$Revision: 1.43 $";
+static const char version[] = "$Revision: 1.59 $";
 
 static const char *known_env_vars[] = {
   "c_include_path",
@@ -130,7 +132,10 @@ add_path (char *s, int maxlen)
     *--e = 0;
   for (int i = 1; i < num_paths; i++)
     if (strcasecmp (paths[num_paths], paths[i]) == 0)
-      return;
+      {
+	free (paths[num_paths]);
+	return;
+      }
   num_paths++;
 }
 
@@ -271,7 +276,7 @@ get_dword (HANDLE fh, int offset)
 
   if (SetFilePointer (fh, offset, 0, FILE_BEGIN) == INVALID_SET_FILE_POINTER
       && GetLastError () != NO_ERROR)
-    keyeprint ("get_word: SetFilePointer()");
+    keyeprint ("get_dword: SetFilePointer()");
 
   if (!ReadFile (fh, &rv, 4, (DWORD *) &r, 0))
     keyeprint ("get_dword: Readfile()");
@@ -354,7 +359,7 @@ cygwin_info (HANDLE h)
   buf_start = buf = (char *) calloc (1, size + 1);
   if (buf == NULL)
     {
-      keyeprint ("cygwin_info: malloc()");
+      keyeprint ("cygwin_info: calloc()");
       return;
     }
 
@@ -532,13 +537,13 @@ track_down (char *file, char *suffix, int lvl)
 {
   if (file == NULL)
     {
-      keyeprint ("track_down: malloc()");
+      keyeprint ("track_down: NULL passed for file");
       return;
     }
 
   if (suffix == NULL)
     {
-      keyeprint ("track_down: malloc()");
+      keyeprint ("track_down: NULL passed for suffix");
       return;
     }
 
@@ -788,60 +793,59 @@ pretty_id (const char *s, char *cygwin, size_t cyglen)
   FILE *f = popen (id, "rt");
 
   char buf[16384];
-  static char empty[] = "";
   buf[0] = '\0';
   fgets (buf, sizeof (buf), f);
+  pclose (f);
   char *uid = strtok (buf, ")");
   if (uid)
     uid += strlen ("uid=");
   else
-    uid = empty;
+    {
+      fprintf (stderr, "garbled output from `id' command - no uid= found\n");
+      return;
+    }
   char *gid = strtok (NULL, ")");
   if (gid)
     gid += strlen ("gid=") + 1;
   else
-    gid = empty;
-  char **ng;
-  size_t sz = 0;
-  for (ng = groups; (*ng = strtok (NULL, ",")); ng++)
+    {
+      fprintf (stderr, "garbled output from `id' command - no gid= found\n");
+      return;
+    }
+
+  char **ng = groups - 1;
+  size_t len_uid = strlen ("UID: )") + strlen (uid);
+  size_t len_gid = strlen ("GID: )") + strlen (gid);
+  *++ng = groups[0] = (char *) alloca (len_uid + 1);
+  *++ng = groups[1] = (char *) alloca (len_gid + 1);
+  sprintf (groups[0], "UID: %s)", uid);
+  sprintf (groups[1], "GID: %s)", gid);
+  size_t sz = max (len_uid, len_gid);
+  while ((*++ng = strtok (NULL, ",")))
     {
       char *p = strchr (*ng, '\n');
       if (p)
 	*p = '\0';
-      if (ng == groups)
+      if (ng == groups + 2)
 	*ng += strlen (" groups=");
       size_t len = strlen (*ng);
       if (sz < len)
 	sz = len;
     }
+  ng--;
 
   printf ("\nOutput from %s (%s)\n", id, s);
-  size_t szmaybe = strlen ("UID: ") + strlen (uid);
-  if (sz < szmaybe)
-    sz = szmaybe;
-  sz += 1;
-  int n = 80 / (int) sz;
+  int n = 80 / (int) ++sz;
+  int i = n > 2 ? n - 2 : 0;
   sz = -sz;
-  ng[0] += strlen ("groups=");
-  printf ("UID: %.*s) GID: %s)\n", sz + (sizeof ("UID: ") - 1), uid, gid);
-  int i = 0;
-  for (char **g = groups; g < ng; g++)
-    {
-      if (i < n)
-	i++;
-      else
-	{
-	  i = 0;
-	  puts ("");
-	}
-      if (++i <= n && g != (ng - 1))
-	printf ("%*s ", sz, *g);
-      else
-	{
-	  printf ("%s\n", *g);
-	  i = 0;
-	}
-    }
+  for (char **g = groups; g <= ng; g++)
+    if ((g != ng) && (++i < n))
+      printf ("%*s", sz, *g);
+    else
+      {
+	puts (*g);
+	i = 0;
+      }
 }
 
 static void
@@ -956,7 +960,10 @@ dump_sysinfo ()
   while (1)
     {
       for (e = s; *e && *e != sep; e++);
-      printf ("\t%.*s\n", e - s, s);
+      if (e-s)
+	printf ("\t%.*s\n", e - s, s);
+      else
+	puts ("\t.");
       count_path_items++;
       if (!*e)
 	break;
@@ -1057,7 +1064,7 @@ dump_sysinfo ()
   if (givehelp)
     {
       printf ("Listing available drives...\n");
-      printf ("Drv Type        Size   Free Flags              Name\n");
+      printf ("Drv Type          Size   Used Flags              Name\n");
     }
   int prev_mode =
     SetErrorMode (SEM_FAILCRITICALERRORS | SEM_NOOPENFILEERRORBOX);
@@ -1076,9 +1083,9 @@ dump_sysinfo ()
       DWORD serno = 0, maxnamelen = 0, flags = 0;
       name[0] = name[0] = fsname[0] = 0;
       sprintf (drive, "%c:\\", i + 'a');
-      /* Report all errors, except if the Volume is ERROR_NOT_READY. 
-         ERROR_NOT_READY is returned when removeable media drives are empty
-         (CD, floppy, etc.) */
+      /* Report all errors, except if the Volume is ERROR_NOT_READY.
+	 ERROR_NOT_READY is returned when removeable media drives are empty
+	 (CD, floppy, etc.) */
       if (!GetVolumeInformation
 	  (drive, name, sizeof (name), &serno, &maxnamelen, &flags, fsname,
 	   sizeof (fsname)) && GetLastError () != ERROR_NOT_READY)
@@ -1128,9 +1135,9 @@ dump_sysinfo ()
 
       printf ("%.2s  %s %-6s ", drive, drive_type, fsname);
       if (capacity_mb >= 0)
-	printf ("%5dMb %3d%% ", (int) capacity_mb, (int) percent_full);
+	printf ("%7dMb %3d%% ", (int) capacity_mb, (int) percent_full);
       else
-	printf ("  N/A    N/A ");
+	printf ("    N/A    N/A ");
       printf ("%s %s %s %s %s %s  %s\n",
 	      flags & FS_CASE_IS_PRESERVED ? "CP" : "  ",
 	      flags & FS_CASE_SENSITIVE ? "CS" : "  ",
@@ -1153,27 +1160,34 @@ dump_sysinfo ()
   SetErrorMode (prev_mode);
   if (givehelp)
     {
-      printf ("fd=floppy, hd=hard drive, cd=CD-ROM, net=Network Share\n");
-      printf ("CP=Case Preserving, CS=Case Sensitive, UN=Unicode\n");
-      printf
-	("PA=Persistent ACLS, FC=File Compression, VC=Volume Compression\n");
+      puts ("\n"
+	  "fd = floppy,          hd = hard drive,       cd = CD-ROM\n"
+	  "net= Network Share,   ram= RAM drive,        unk= Unknown\n"
+	  "CP = Case Preserving, CS = Case Sensitive,   UN = Unicode\n"
+	  "PA = Persistent ACLS, FC = File Compression, VC = Volume Compression");
     }
   printf ("\n");
 
   unsigned ml_fsname = 4, ml_dir = 7, ml_type = 6;
+  bool ml_trailing = false;
 
   struct mntent *mnt;
   setmntent (0, 0);
   while ((mnt = getmntent (0)))
     {
       unsigned n = (int) strlen (mnt->mnt_fsname);
+      ml_trailing |= (n > 1 && strchr ("\\/", mnt->mnt_fsname[n - 1]));
       if (ml_fsname < n)
 	ml_fsname = n;
       n = (int) strlen (mnt->mnt_dir);
+      ml_trailing |= (n > 1 && strchr ("\\/", mnt->mnt_dir[n - 1]));
       if (ml_dir < n)
 	ml_dir = n;
     }
 
+  if (ml_trailing)
+    puts ("Warning: Mount entries should not have a trailing (back)slash\n");
+
   if (givehelp)
     {
       printf
@@ -1208,6 +1222,7 @@ dump_sysinfo ()
 
   if (givehelp)
     printf ("Looking for various Cygnus DLLs...  (-v gives version info)\n");
+  int cygwin_dll_count = 0;
   for (i = 0; i < num_paths; i++)
     {
       WIN32_FIND_DATA ffinfo;
@@ -1224,7 +1239,10 @@ dump_sysinfo ()
 		{
 		  sprintf (tmp, "%s\\%s", paths[i], f);
 		  if (strcasecmp (f, "cygwin1.dll") == 0)
-		    found_cygwin_dll = strdup (tmp);
+		    {
+		      cygwin_dll_count++;
+		      found_cygwin_dll = strdup (tmp);
+		    }
 		  else
 		    ls (tmp);
 		}
@@ -1239,6 +1257,10 @@ dump_sysinfo ()
 
       FindClose (ff);
     }
+  if (cygwin_dll_count > 1)
+    puts ("Warning: There are multiple cygwin1.dlls on your path");
+  if (!cygwin_dll_count)
+    puts ("Warning: cygwin1.dll not found on your path");
 }
 
 static int
@@ -1249,7 +1271,7 @@ check_keys ()
 			  OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 
   if (h == INVALID_HANDLE_VALUE || h == NULL)
-    return (keyeprint ("check_key: Opening CONIN$"));
+    return (keyeprint ("check_keys: Opening CONIN$"));
 
   DWORD mode;
 
@@ -1259,7 +1281,7 @@ check_keys ()
     {
       mode &= ~ENABLE_PROCESSED_INPUT;
       if (!SetConsoleMode (h, mode))
-	keyeprint ("check_keys: GetConsoleMode()");
+	keyeprint ("check_keys: SetConsoleMode()");
     }
 
   fputs ("\nThis key check works only in a console window,", stderr);
@@ -1278,7 +1300,7 @@ check_keys ()
     {
       prev_in = in;
       if (!ReadConsoleInput (h, &in, 1, &mode))
-	keyeprint ("ReadConsoleInput");
+	keyeprint ("check_keys: ReadConsoleInput()");
 
       if (!memcmp (&in, &prev_in, sizeof in))
 	continue;
