@@ -51,10 +51,21 @@ extern "C" {
 #define NIM_SETFOCUS	3
 #define NIM_SETVERSION	4
 #endif
-#define NIF_MESSAGE	1
-#define NIF_ICON	2
-#define NIF_TIP	4
-#define NIF_STATE	8
+#define NIF_MESSAGE	0x00000001
+#define NIF_ICON	0x00000002
+#define NIF_TIP		0x00000004
+#define NIF_STATE	0x00000008
+#if (_WIN32_IE >= 0x0500)
+#define NIF_INFO	0x00000010
+#define NIIF_NONE	0x00000000
+#define NIIF_INFO	0x00000001
+#define NIIF_WARNING	0x00000002
+#define NIIF_ERROR	0x00000003
+#endif
+#if (_WIN32_IE >= 0x0600)
+#define NIIF_ICON_MASK	0x0000000F
+#define NIIF_NOSOUND	0x00000010
+#endif
 #define NIS_HIDDEN	1
 #define NIS_SHAREDICON	2
 #define SE_ERR_FNF	2
@@ -254,6 +265,7 @@ void WINAPI DragFinish(HDROP);
 UINT WINAPI DragQueryFileA(HDROP,UINT,LPSTR,UINT);
 UINT WINAPI DragQueryFileW(HDROP,UINT,LPWSTR,UINT);
 BOOL WINAPI DragQueryPoint(HDROP,LPPOINT);
+HICON WINAPI DuplicateIcon(HINSTANCE,HICON);
 HICON WINAPI ExtractAssociatedIconA(HINSTANCE,LPCSTR,PWORD);
 HICON WINAPI ExtractAssociatedIconW(HINSTANCE,LPCWSTR,PWORD);
 HICON WINAPI ExtractIconA(HINSTANCE,LPCSTR,UINT);
